@@ -2,7 +2,7 @@
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
- * Copyright (C) Richard P. Curnow  2002,2003,2004,2005
+ * Copyright (C) Richard P. Curnow  2002,2003,2004,2005,2007
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -857,7 +857,7 @@ static int lookup_msgpath(struct msgpath *sorted_paths, int n_msgs, char *key)/*
 void maybe_grow_message_arrays(struct database *db)/*{{{*/
 {
   if (db->n_msgs == db->max_msgs) {
-    if (db->max_msgs == 0) {
+    if (db->max_msgs <= 128) {
       db->max_msgs = 256;
     } else {
       db->max_msgs += (db->max_msgs >> 1);
